@@ -494,7 +494,7 @@ NumericVector fe2d(NumericVector cv1, NumericVector cv2, NumericVector width1, N
     dcv1 = cv1[i]-x;
     dcv2 = cv2[i]-y;
     v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]);
-    vo(i)=v;
+    vo(i-tmin)=v;
   }
   return vo;
 }
@@ -512,7 +512,7 @@ NumericVector fe2dp1(NumericVector cv1, NumericVector cv2, NumericVector width1,
     if(dcv1 < -p1/2.0) dcv1 += p1;
     dcv2 = cv2[i]-y;
     v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]);
-    vo(i)=v;
+    vo(i-tmin)=v;
   }
   return vo;
 }
@@ -530,7 +530,7 @@ NumericVector fe2dp2(NumericVector cv1, NumericVector cv2, NumericVector width1,
     if(dcv2 >  p2/2.0) dcv2 -= p2;
     if(dcv2 < -p2/2.0) dcv2 += p2;
     v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]);
-    vo(i)=v;
+    vo(i-tmin)=v;
   }
   return vo;
 }
@@ -550,7 +550,7 @@ NumericVector fe2dp12(NumericVector cv1, NumericVector cv2, NumericVector width1
     if(dcv2 >  p2/2.0) dcv2 -= p2;
     if(dcv2 < -p2/2.0) dcv2 += p2;
     v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]);
-    vo(i)=v;
+    vo(i-tmin)=v;
   }
   return vo;
 }
@@ -564,7 +564,7 @@ NumericVector fe1d(NumericVector cv1, NumericVector width1, NumericVector height
   for (int i=tmin; i <= tmax; i++) {
     dcv1 = cv1[i]-x;
     v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]);
-    vo(i)=v;
+    vo(i-tmin)=v;
   }
   return vo;
 }
@@ -580,8 +580,98 @@ NumericVector fe1dp(NumericVector cv1, NumericVector width1, NumericVector heigh
     if(dcv1 >  p1/2.0) dcv1 -= p1;
     if(dcv1 < -p1/2.0) dcv1 += p1;
     v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]);
-    vo(i)=v;
+    vo(i-tmin)=v;
   }
   return vo;
 }
 
+// [[Rcpp::export]]
+double f2d(NumericVector cv1, NumericVector cv2, NumericVector width1, NumericVector width2, NumericVector heights, double x, double y, int tmin, int tmax) {
+  double dcv1;
+  double dcv2;
+  double v;
+  v = 0.0;
+  for (int i=tmin; i <= tmax; i++) {
+    dcv1 = cv1[i]-x;
+    dcv2 = cv2[i]-y;
+    v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]);
+  }
+  return v;
+}
+
+// [[Rcpp::export]]
+double f2dp1(NumericVector cv1, NumericVector cv2, NumericVector width1, NumericVector width2, NumericVector heights, double x, double y, double p1, int tmin, int tmax) {
+  double dcv1;
+  double dcv2;
+  double v;
+  v = 0.0;
+  for (int i=tmin; i <= tmax; i++) {
+    dcv1 = cv1[i]-x;
+    if(dcv1 >  p1/2.0) dcv1 -= p1;
+    if(dcv1 < -p1/2.0) dcv1 += p1;
+    dcv2 = cv2[i]-y;
+    v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]);
+  }
+  return v;
+}
+
+// [[Rcpp::export]]
+double f2dp2(NumericVector cv1, NumericVector cv2, NumericVector width1, NumericVector width2, NumericVector heights, double x, double y, double p2, int tmin, int tmax) {
+  double dcv1;
+  double dcv2;
+  double v;
+  v = 0.0;
+  for (int i=tmin; i <= tmax; i++) {
+    dcv1 = cv1[i]-x;
+    dcv2 = cv2[i]-y;
+    if(dcv2 >  p2/2.0) dcv2 -= p2;
+    if(dcv2 < -p2/2.0) dcv2 += p2;
+    v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]);
+  }
+  return v;
+}
+
+// [[Rcpp::export]]
+double f2dp12(NumericVector cv1, NumericVector cv2, NumericVector width1, NumericVector width2, NumericVector heights, double x, double y, double p1, double p2, int tmin, int tmax) {
+  double dcv1;
+  double dcv2;
+  double v;
+  v = 0.0;
+  for (int i=tmin; i <= tmax; i++) {
+    dcv1 = cv1[i]-x;
+    if(dcv1 >  p1/2.0) dcv1 -= p1;
+    if(dcv1 < -p1/2.0) dcv1 += p1;
+    dcv2 = cv2[i]-y;
+    if(dcv2 >  p2/2.0) dcv2 -= p2;
+    if(dcv2 < -p2/2.0) dcv2 += p2;
+    v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]-dcv2*dcv2/2.0/width2[i]/width2[i]);
+  }
+  return v;
+}
+
+// [[Rcpp::export]]
+double f1d(NumericVector cv1, NumericVector width1, NumericVector heights, double x, int tmin, int tmax) {
+  double dcv1;
+  double v;
+  v = 0.0;
+  for (int i=tmin; i <= tmax; i++) {
+    dcv1 = cv1[i]-x;
+    v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]);
+  }
+  return v;
+}
+
+// [[Rcpp::export]]
+double f1dp(NumericVector cv1, NumericVector width1, NumericVector heights, double x, double p1, int tmin, int tmax) {
+  double dcv1;
+  double v;
+  v = 0.0;
+  for (int i=tmin; i <= tmax; i++) {
+    dcv1 = cv1[i]-x;
+    if(dcv1 >  p1/2.0) dcv1 -= p1;
+    if(dcv1 < -p1/2.0) dcv1 += p1;
+    v -= heights[i]*exp(-dcv1*dcv1/2.0/width1[i]/width1[i]);
+  }
+  return v;
+}
+
